//
//  This file is part of SienaXML, an XML interface to the 
//  Siena wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Author: Chris Corliss <corliss@falcon.stortek.com>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2001-2002 University of Colorado
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: DOMTreeProcessing.java,v 1.1.1.1 2002/06/07 22:42:37 jgiacomo Exp $
//
/*
 * DOMTreeProcessing.java
 *
 * Created on December 14, 2001, 2:15 PM
 */

package siena.xml;

import org.w3c.dom.Node;
import org.w3c.dom.Attr;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;
import org.apache.xerces.dom.CoreDocumentImpl;
import org.apache.xerces.dom.ElementImpl;

/** This class is used to handle different types of processing of the XML DOM tree
 * generated from the DOMParserWrapper.
 *
 * @author CorliC
 * @version 1.0
 */
public class DOMTreeProcessing {
    
    /** This variable is used to hold the Core Document node used to build children nodes from.
     */    
    private static final CoreDocumentImpl document = new CoreDocumentImpl();
    
    /** This function will find a child node of the node passed in based on
     * the name given it as an arguement. If a child node is not found null is returned.
     * @param name The name of the child node to look for
     * @param node The node to use as the parent node.
     * @return A node that is the child of the node passed in that has the specified name or null if no child could be found.
     */    
    public static Node findChildByName(String name, Node node) {
        String nodeName = null;
        NodeList children = node.getChildNodes();
        for (int i = 0; i < children.getLength(); i++) {
            Node child = children.item(i);
            nodeName = child.getLocalName();
            if (nodeName != null && nodeName.equals(name)) {
                return child;
            }
        }
        return null;
    }
    
    /** This method will return a count of the number of nodes that have the
     * same tag as the node that was passed in.
     * @param node Node to use as the basis of search, both for the tag name and the level in the DOM tree to look at
     * @return A count of the number of nodes that have the same tag and are siblings of the node that was passed in.
     */    
    public static int countSameTagSiblings(Node node) {
        return countSiblingsWithTag(node, node.getLocalName());
    }
    
    /** This function will count the number of siblings to the node passed in that have the same tag name as the one contained in the string.
     * @param node This is the node used as the reference point for the search.
     * @param tagName This is the name of the tag being looked for
     * @return The number of siblings that have the "tagName" for their XML tag
     */    
    public static int countSiblingsWithTag(Node node, String tagName) {
        int counter = 0;
        String testTag = null;
        Node temp = node.getPreviousSibling();
        
        while (temp != null) {
            testTag = temp.getLocalName();
            if (testTag != null && testTag.equals(tagName)) {
                counter++;
            }
            temp = temp.getPreviousSibling();
        }
        temp = node.getNextSibling();
        while (temp != null) {
            testTag = temp.getLocalName();
            if (testTag != null && testTag.equals(tagName)) {
                counter++;
            }
            temp = temp.getNextSibling();
        }
        
        return counter;
    }
    
    /**
     * This method will get the value for the xml tag represented by the node
     * passed in. If the node has no value but the first child of the node is a
     * text node then the value of the text node is returned.
     *
     * @param node The node to get the value for
     * @return A string containing the value. If there is no value then null is
     *         returned.
     */
    
    public static String getNodeValue(Node node) {
        String value = node.getNodeValue();
        if (value == null) {
            Node childNode = node.getFirstChild();
            if (childNode != null && childNode.getNodeType() == Node.TEXT_NODE) {
                value = childNode.getNodeValue();
            }
        }
        return value;
    }
    
    /** Count the number of siblings that have the same attribute as the node passed in.
     * @param node The reference Node that determines the attribute value and what nodes to look at (its siblings)
     * @param targetAttr The name of the attribute to look for in the sibling nodes
     * @return The number of sibling nodes that have the same value for the "targetAttr" attribute.
     */    
    public static int countSameAttributeValueSiblings(Node node, String targetAttr) {
        int counter = 0;
        String targetValue = null;
        Node temp = null;
        Node attrTemp = null;
        org.w3c.dom.NamedNodeMap attributes = null;
        
        targetValue = node.getAttributes().getNamedItem(targetAttr).getNodeValue();
        
        temp = node.getPreviousSibling();
        while (temp != null) {
            attributes = temp.getAttributes();
            if (attributes != null) {
                attrTemp = attributes.getNamedItem(targetAttr);                
                if (attrTemp != null && attrTemp.getNodeValue().equals(targetValue)) {
                    counter++;
                }
            }
            temp = temp.getPreviousSibling();
        }
        
        temp = node.getNextSibling();
        while (temp != null) {
            attributes = temp.getAttributes();
            if (attributes != null) {
                attrTemp = attributes.getNamedItem(targetAttr);
                if (attrTemp != null && attrTemp.getNodeValue().equals(targetValue)) {
                    counter++;
                }
            }
            temp = temp.getNextSibling();
        }
        
        return counter;
    }

    /** Create a node with the name <i>name</i> and with the value <i>value</i>
     * @param name The name that the new Node should have
     * @param value The value that the new Node should have
     * @return The created Node
     */    
    public static Node createNode(String name, String value) {
        return createNode(name,value, "");
    }

    /** Create a node that has the name <i>name</i>, value of <i>value</>, in the namespace <i>namespace</i>
     * @param name Name of the new Node
     * @param value Value of the new Node
     * @param namespace Namespace which the node should be in
     * @return The created Node
     */    
    public static Node createNode(String name, String value, String namespace) {
        ElementImpl node = (ElementImpl)document.createElementNS(namespace,name);
        if (value != null) {
            node.appendChild(document.createTextNode(value));
        }        
        return node;
    }
    
    /** This sets the value for a node to <i>value</i>
     * @param node The Node object to change the value of
     * @param value The new value to assign to the Node
     */    
    public static void setValue(Node node, String value) {
        node.appendChild(document.createTextNode(value));
    }
}
